[LTP] [PATCH v4] syscalls/xattr: Add runtime probe for socket xattr backports

Darren Chang chihsheng@google.com
Tue Jun 2 03:07:54 CEST 2026


Upstream commit dc0876b9846d ("xattr: support extended attributes on
sockets") was merged in v7.1. Currently, setxattr02 and fsetxattr02
strictly expect EPERM for sockets on kernels < 7.1.

However, downstream kernels (e.g., Android mainline branches) often
backport this feature to older kernel versions (such as 7.0). On
these kernels, the strict version check causes a false positive failure:
"passed unexpectedly".

This patch introduces a runtime probe in setup() for both tests to
detect if older kernels have the backported feature. If the xattr
syscalls succeed on a socket during setup, we assume the patch is
present and adjust the expected result to pass.

Signed-off-by: Darren Chang <chihsheng@google.com>
---
 .../kernel/syscalls/fsetxattr/fsetxattr02.c   | 34 ++++++++++++++++---
 .../kernel/syscalls/setxattr/setxattr02.c     | 14 ++++++++
 2 files changed, 44 insertions(+), 4 deletions(-)

diff --git a/testcases/kernel/syscalls/setxattr/setxattr02.c b/testcases/kernel/syscalls/setxattr/setxattr02.c
index 994ab655d..167e068cf 100644
--- a/testcases/kernel/syscalls/setxattr/setxattr02.c
+++ b/testcases/kernel/syscalls/setxattr/setxattr02.c
@@ -196,6 +196,20 @@ static void setup(void)
 	SAFE_MKNOD(SOCK, S_IFSOCK | 0777, 0);
 
 	socket_xattr_supported = tst_kvercmp(7, 1, 0) >= 0;
+
+	/*
+	 * Commit dc0876b9846d "xattr: support extended attributes on sockets"
+	 * merged in 7.1 mainline, but might be backported.
+	 * Try to set xattr on socket to detect support.
+	 */
+	if (!socket_xattr_supported) {
+		if (setxattr(SOCK, XATTR_TEST_KEY, XATTR_TEST_VALUE,
+			     XATTR_TEST_VALUE_SIZE, XATTR_CREATE) == 0) {
+			tst_res(TINFO, "Socket xattr support detected (backport)");
+			socket_xattr_supported = true;
+			SAFE_REMOVEXATTR(SOCK, XATTR_TEST_KEY);
+		}
+	}
 }
 
 static struct tst_test test = {
diff --git a/testcases/kernel/syscalls/fsetxattr/fsetxattr02.c b/testcases/kernel/syscalls/fsetxattr/fsetxattr02.c
index 2a48b7da7..4a0997445 100644
--- a/testcases/kernel/syscalls/fsetxattr/fsetxattr02.c
+++ b/testcases/kernel/syscalls/fsetxattr/fsetxattr02.c
@@ -19,10 +19,10 @@
  * - Set attribute to a block special file, fsetxattr(2) should
  *   return -1 and set errno to EPERM.
  * - Set attribute to a UNIX domain socket, fsetxattr(2) should
  *   return -1 and set errno to EPERM on kernels < 7.1.0.
  *   On kernel 7.1.0+ (dc0876b9846d "xattr: support extended
  *   attributes on sockets"), returns 0 (success) as sockets now
  *   support user.* xattrs.
  */
 
 /*
@@ -209,6 +209,7 @@ static void verify_fsetxattr(unsigned int i)
 static void setup(void)
 {
 	size_t i = 0;
+	size_t socket_idx = ARRAY_SIZE(tc);
 	struct sockaddr_un sun;
 
 	dev_t dev = makedev(1, 3);
@@ -245,6 +246,31 @@ static void setup(void)
 	}
 
 	socket_xattr_supported = tst_kvercmp(7, 1, 0) >= 0;
+
+	/*
+	 * Commit dc0876b9846d "xattr: support extended attributes on sockets"
+	 * merged in 7.1 mainline, but might be backported.
+	 * Try to set xattr on socket to detect support.
+	 */
+	if (!socket_xattr_supported) {
+		for (i = 0; i < ARRAY_SIZE(tc); i++) {
+			if (tc[i].issocket) {
+				socket_idx = i;
+				break;
+			}
+		}
+
+		if (socket_idx != ARRAY_SIZE(tc)) {
+			if (fsetxattr(tc[socket_idx].fd, tc[socket_idx].key,
+				      tc[socket_idx].value, tc[socket_idx].size,
+				      tc[socket_idx].flags) == 0) {
+				tst_res(TINFO, "Socket xattr support detected (backport)");
+				socket_xattr_supported = true;
+
+				SAFE_FREMOVEXATTR(tc[socket_idx].fd, tc[socket_idx].key);
+			}
+		}
+	}
 }
 
 static void cleanup(void)
@@ -19,10 +19,10 @@
  *   return -1 and set errno to EPERM.
  * - Set attribute to a UNIX domain socket, fsetxattr(2) should
- *   return -1 and set errno to EPERM on kernels < 7.1.0.
- *   On kernel 7.1.0+ (dc0876b9846d "xattr: support extended
- *   attributes on sockets"), returns 0 (success) as sockets now
- *   support user.* xattrs.
+ *   return -1 and set errno to EPERM on kernels < 7.1, unless the
+ *   feature is detected at runtime (e.g. backport). On kernel 7.1+
+ *   (dc0876b9846d "xattr: support extended attributes on sockets"),
+ *   or when the backport is probed in setup(), returns 0 (success).
  */
 
-- 
2.54.0.929.g9b7fa37559-goog



More information about the ltp mailing list